/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Gateway.
 *
 * The Initial Developer of the Original Code is
 * the Vermont Department of Taxes.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Tom Cort <tom.cort@state.vt.us>
 *
 * ***** END LICENSE BLOCK ***** */

package net.sf.mavenhar;

import java.io.File;

import org.apache.maven.archiver.MavenArchiveConfiguration;
import org.apache.maven.archiver.MavenArchiver;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

import org.apache.maven.project.MavenProject;
import org.apache.maven.project.MavenProjectHelper;

import org.codehaus.plexus.archiver.jar.JarArchiver;

/**
 * Creates a Hibernate Archive.
 * 
 * @goal har
 * @phase package
 * @requiresDependencyResolution runtime
 */
public class HarMojo extends AbstractMojo {

    /**
     * The Project.
     * 
     * @parameter expression="${project}"
     * @required
     * @readonly
     */
    private MavenProject project;

    /**
     * The Project Helper.
     * 
     * @component
     */
    private MavenProjectHelper helper;

    /**
     * Archive Configuration.
     * 
     * @parameter
     */
    private MavenArchiveConfiguration archiveConfig = new MavenArchiveConfiguration();

    /**
     * The JAR Archiver.
     * 
     * @parameter expression="${component.org.codehaus.plexus.archiver.Archiver#jar}"
     * @required
     */
    private JarArchiver jarArchiver;

    /**
     * Directory where the har file should be written to.
     * 
     * @parameter expression="${project.build.directory}"
     * @required
     */
    private File targetDir;

    /**
     * Directory containing the classes (usually target/classes).
     * 
     * @parameter expression="${project.build.outputDirectory}"
     * @required
     */
    private File classesDir;

    /**
     * Name of the generated HAR (without the ".har" extension).
     * 
     * @parameter expression="${project.build.finalName}"
     * @required
     */
    private String targetFile;

    /**
     * Classifier to add to the artifact generated.
     * 
     * @parameter
     */
    private String classifier;

    /**
     * List of files to exclude from the HAR.
     */
    private static final String[] EXCLUDES = new String[] { "**/*.html" };

    /**
     * List of files in include in the HAR.
     */
    private static final String[] INCLUDES = new String[] { "META-INF/jboss-service.xml", "**/*.hbm.xml", "**/*.class" };

    /**
     * Executes the task. In this case, packages the har.
     * 
     * @throws MojoExecutionException
     *         on archiver error.
     */
    public final void execute() throws MojoExecutionException {

        getLog().info("====== MAVEN-HAR-PLUGIN ======");
        getLog().info("targetDir  [" + targetDir + "]");
        getLog().info("targetFile [" + targetFile + ".har]");
        getLog().info("classesDir [" + classesDir.toString() + "]");

        File har = new File(targetDir, targetFile + ".har");

        MavenArchiver archiver = new MavenArchiver();
        archiver.setArchiver(jarArchiver);
        archiver.setOutputFile(har);

        try {

            // Attempt to locale jboss-service.xml, fail if not found.
            File jbossServiceXml = new File(classesDir.toString() + "/META-INF/jboss-service.xml");

            if (!jbossServiceXml.exists()) {
                String msg = classesDir.toString() + "/META-INF/jboss-service.xml was not found.";
                throw new MojoExecutionException(msg);
            }

            archiver.getArchiver().addDirectory(classesDir, INCLUDES, EXCLUDES);
            archiver.createArchive(project, archiveConfig);

            if (classifier == null) {
                project.getArtifact().setFile(har);
            } else {
                helper.attachArtifact(project, "har", classifier, har);
            }

        } catch (Exception e) {
            throw new MojoExecutionException("Error packaging HAR", e);
        }

        getLog().info("====== MAVEN-HAR-PLUGIN ======");
    }
}
